@@ -70,7 +70,7 @@ struct _GogRenderer {
 
 	GogGraph *model;
 	GogView	 *view;
-	float	  scale, scale_x, scale_y;
+	double	  scale, scale_x, scale_y;
 
 	GClosure *font_watcher;
 	gboolean  needs_update;
@@ -79,7 +79,6 @@ struct _GogRenderer {
 	GSList   *style_stack;
 
 	GOLineDashSequence 	*line_dash;
-	GOLineDashSequence 	*outline_dash;
 
 	GOStyle 	*grip_style;
 	GOStyle 	*selection_style;
@@ -131,7 +130,7 @@ _line_size (GogRenderer const *rend, double width, gboolean sharp)
 {
 	double size = _grc_line_size (rend, width, sharp);
 
-	if (!rend->is_vector && size < 1.0)
+	if (!rend->is_vector && sharp && size < 1.0)
 		return ceil (size);
 
 	return size;
@@ -144,16 +143,12 @@ _update_dash (GogRenderer *rend)
 
 	go_line_dash_sequence_free (rend->line_dash);
 	rend->line_dash = NULL;
-	go_line_dash_sequence_free (rend->outline_dash);
-	rend->outline_dash = NULL;
 
 	if (rend->cur_style == NULL)
 		return;
 
 	size = _line_size (rend, rend->cur_style->line.width, FALSE);
 	rend->line_dash = go_line_dash_get_sequence (rend->cur_style->line.dash_type, size);
-	size = _line_size (rend, rend->cur_style->outline.width, FALSE);
-	rend->outline_dash = go_line_dash_get_sequence (rend->cur_style->outline.dash_type, size);
 }
 
 double
@@ -215,39 +210,6 @@ emit_line (GogRenderer *rend, gboolean preserve, GOPathOptions options)
 		cairo_stroke (cr);
 }
 
-static void
-emit_outline (GogRenderer *rend, gboolean preserve, GOPathOptions options)
-{
-	GOStyle const *style = rend->cur_style;
-	cairo_t *cr = rend->cairo;
-	double width;
-
-	if (!go_style_is_outline_visible (style)) {
-		if (!preserve)
-			cairo_new_path (cr);
-		return;
-	}
-
-	width = _grc_line_size (rend, style->outline.width, options & GO_PATH_OPTIONS_SNAP_WIDTH);
-	cairo_set_line_width (cr, width);
-	if (rend->outline_dash != NULL)
-		cairo_set_dash (cr,
-				rend->outline_dash->dash,
-				rend->outline_dash->n_dash,
-				rend->outline_dash->offset);
-	else
-		cairo_set_dash (cr, NULL, 0, 0);
-
-	cairo_set_source_rgba (cr, GO_COLOR_TO_CAIRO (style->outline.color));
-	cairo_set_line_cap (cr, (width <= 2.0 && !rend->is_vector) ?
-			    CAIRO_LINE_CAP_SQUARE : CAIRO_LINE_CAP_ROUND);
-
-	if (preserve)
-		cairo_stroke_preserve (cr);
-	else
-		cairo_stroke (cr);
-}
-
 static void
 emit_fill (GogRenderer *rend, gboolean preserve)
 {
@@ -435,7 +397,6 @@ gog_renderer_stroke_serie (GogRenderer *renderer,
 {
 	GOStyle const *style;
 	GOPathOptions line_options;
-	gboolean is_outline;
 	double width;
 
 	g_return_if_fail (GOG_IS_RENDERER (renderer));
@@ -443,18 +404,14 @@ gog_renderer_stroke_serie (GogRenderer *renderer,
 	g_return_if_fail (GO_IS_PATH (path));
 
         style = renderer->cur_style;
-	is_outline = style->interesting_fields & GO_STYLE_OUTLINE;
 	line_options = go_path_get_options (path);
 	width = _grc_line_size (renderer,
-				is_outline ?  style->outline.width : style->line.width,
+				style->line.width,
 				line_options & GO_PATH_OPTIONS_SNAP_WIDTH);
 
 	if (go_style_is_line_visible (style)) {
 		path_interpret (renderer, path, width);
-		if (is_outline)
-			emit_outline (renderer, FALSE, go_path_get_options (path));
-		else
-			emit_line (renderer, FALSE, go_path_get_options (path));
+		emit_line (renderer, FALSE, go_path_get_options (path));
 	}
 }
 
@@ -483,17 +440,15 @@ _draw_shape (GogRenderer *renderer, GOPath const *path, gboolean fill, gboolean
 	GOStyle const *style;
 	GOPathOptions line_options;
 	double width;
-	gboolean use_outline;
 
 	g_return_if_fail (GOG_IS_RENDERER (renderer));
 	g_return_if_fail (renderer->cur_style != NULL);
 	g_return_if_fail (GO_IS_PATH (path));
 
         style = renderer->cur_style;
-	use_outline = style->interesting_fields & GO_STYLE_OUTLINE;
 
 	line_options = go_path_get_options (path);
-	width = stroke ? _grc_line_size (renderer, use_outline ? style->outline.width : style->line.width,
+	width = stroke ? _grc_line_size (renderer, style->line.width,
 					 line_options & GO_PATH_OPTIONS_SNAP_WIDTH) : 0;
 
 	path_interpret (renderer, path, width);
@@ -501,12 +456,8 @@ _draw_shape (GogRenderer *renderer, GOPath const *path, gboolean fill, gboolean
 	if (fill)
 		emit_fill (renderer, stroke);
 
-	if (stroke) {
-		if (use_outline)
-			emit_outline (renderer, FALSE, go_path_get_options (path));
-		else
-			emit_line (renderer, FALSE, go_path_get_options (path));
-	}
+	if (stroke)
+		emit_line (renderer, FALSE, go_path_get_options (path));
 }
 
 void
@@ -535,7 +486,7 @@ gog_renderer_fill_shape (GogRenderer *renderer, GOPath const *path)
  * @rend: #GogRenderer
  * @path: a #GOPath
  *
- * Defines the current clipping region. 
+ * Defines the current clipping region.
  **/
 
 void
@@ -559,7 +510,7 @@ gog_renderer_push_clip (GogRenderer *rend, GOPath const *path)
  * @w: width of clipping rectangle
  * @h: height of clipping rectangle
  *
- * Defines a rectangular clipping region. For efficient screen rendering, 
+ * Defines a rectangular clipping region. For efficient screen rendering,
  * this function takes care to round the coordinates.
  **/
 
@@ -615,7 +566,7 @@ _draw_circle (GogRenderer *rend, double x, double y, double r, gboolean fill, gb
 	go_path_set_options (path, GO_PATH_OPTIONS_SHARP);
 
 	if (!narrow) {
-		o = gog_renderer_line_size (rend, style->outline.width);
+		o = gog_renderer_line_size (rend, style->line.width);
 		o_2 = o / 2.;
 	} else
 		o = o_2 = 0.;
@@ -672,7 +623,7 @@ _draw_rectangle (GogRenderer *rend, GogViewAllocation const *rect, gboolean fill
 	go_path_set_options (path, GO_PATH_OPTIONS_SHARP);
 
 	if (!narrow) {
-		o = gog_renderer_line_size (rend, style->outline.width);
+		o = gog_renderer_line_size (rend, style->line.width);
 		o_2 = o / 2.;
 	} else
 		o = o_2 = 0.;
@@ -720,9 +671,9 @@ gog_renderer_draw_grip (GogRenderer *renderer, double x, double y)
 		GOStyle *style;
 
 		style = go_style_new ();
-		style->outline.dash_type = GO_LINE_SOLID;
-		style->outline.width = 0.0;
-		style->outline.color =
+		style->line.dash_type = GO_LINE_SOLID;
+		style->line.width = 0.0;
+		style->line.color =
 		style->fill.pattern.back = 0xff000080;
 		style->fill.pattern.pattern = GO_PATTERN_SOLID;
 		style->fill.type = GO_STYLE_FILL_PATTERN;
@@ -749,9 +700,9 @@ gog_renderer_draw_selection_rectangle (GogRenderer *renderer, GogViewAllocation
 		GOStyle *style;
 
 		style = go_style_new ();
-		style->outline.dash_type = GO_LINE_DOT;
-		style->outline.width = 0.0;
-		style->outline.color = 0x0000ffB0;
+		style->line.dash_type = GO_LINE_DOT;
+		style->line.width = 0.0;
+		style->line.color = 0x0000ffB0;
 		style->fill.type = GO_STYLE_FILL_NONE;
 		style->interesting_fields = GO_STYLE_OUTLINE;
 
@@ -1078,11 +1029,12 @@ gog_renderer_update (GogRenderer *rend, double w, double h)
 
 	rend->cairo = cairo_create (rend->cairo_surface);
 
-	if (size_changed) {
-		rend->scale_x = (graph->width >= 1.) ? (rend->w / graph->width) : 1.;
-		rend->scale_y = (graph->height >= 1.) ? (rend->h / graph->height) : 1.;
-		rend->scale = MIN (rend->scale_x, rend->scale_y);
+	/* we need to update scale even if size did not change since graph size might have changed (#599887) */
+	rend->scale_x = (graph->width >= 1.) ? (rend->w / graph->width) : 1.;
+	rend->scale_y = (graph->height >= 1.) ? (rend->h / graph->height) : 1.;
+	rend->scale = MIN (rend->scale_x, rend->scale_y);
 
+	if (size_changed) {
 		/* make sure we dont try to queue an update while updating */
 		rend->needs_update = TRUE;
 
@@ -1124,7 +1076,7 @@ gog_renderer_update (GogRenderer *rend, double w, double h)
 	return redraw;
 }
 
-/** 
+/**
  * gog_renderer_get_pixbuf:
  * @renderer : #GogRenderer
  *
@@ -1241,7 +1193,7 @@ gog_renderer_render_to_cairo (GogRenderer *renderer, cairo_t *cairo, double widt
  * @y_dpi: y resolution of exported graph
  *
  * Exports an image of @graph in given @format, writing results in a #GsfOutput stream.
- * If export format type is a bitmap one, it computes image size with x_dpi, y_dpi and 
+ * If export format type is a bitmap one, it computes image size with x_dpi, y_dpi and
  * @graph size (see gog_graph_get_size()).
  *
  * returns: %TRUE if export succeed.
@@ -1255,9 +1207,12 @@ gog_renderer_export_image (GogRenderer *rend, GOImageFormat format,
 	cairo_t *cairo;
 	cairo_surface_t *surface = NULL;
 	gboolean status;
-	GdkPixbuf *pixbuf, *output_pixbuf;
-	double width_in_pts, height_in_pts;
+	GdkPixbuf *pixbuf;
+#ifdef GOFFICE_WITH_GTK
+	GdkPixbuf *output_pixbuf;
 	gboolean result;
+#endif
+	double width_in_pts, height_in_pts;
 
 	g_return_val_if_fail (GOG_IS_RENDERER (rend), FALSE);
 
@@ -1399,18 +1354,29 @@ void
 gog_renderer_draw_equation (GogRenderer *renderer, LsmMathmlView *mathml_view, double x, double y)
 {
 	cairo_t *cairo;
+	double w, h, alpha;
 
 	g_return_if_fail (GOG_IS_RENDERER (renderer));
 	g_return_if_fail (LSM_IS_MATHML_VIEW (mathml_view));
+	g_return_if_fail (renderer->cur_style != NULL);
 
 	cairo = renderer->cairo;
 
+	alpha = -renderer->cur_style->text_layout.angle * M_PI / 180.0;
+	lsm_dom_view_get_size (LSM_DOM_VIEW (mathml_view), &w, &h);
+	w *= renderer->scale;
+	h *= renderer->scale;
+	x = x - (w / 2.0) * cos (alpha) + (h / 2.0) * sin (alpha);
+	y = y - (w / 2.0) * sin (alpha) - (h / 2.0) * cos (alpha);
+
 	cairo_save (cairo);
 
+	cairo_translate (cairo, x, y);
+	cairo_rotate (cairo, alpha);
 	cairo_scale (cairo, renderer->scale_x, renderer->scale_y);
 
 	lsm_dom_view_set_cairo (LSM_DOM_VIEW (mathml_view), cairo);
-	lsm_dom_view_render (LSM_DOM_VIEW (mathml_view), x / renderer->scale_x, y / renderer->scale_y);
+	lsm_dom_view_render (LSM_DOM_VIEW (mathml_view), 0., 0.);
 
 	cairo_restore (cairo);
 }
@@ -1449,7 +1415,6 @@ gog_renderer_init (GogRenderer *rend)
 	rend->is_vector = FALSE;
 
 	rend->line_dash = NULL;
-	rend->outline_dash = NULL;
 
 	rend->needs_update = FALSE;
 	rend->cur_style    = NULL;
@@ -1483,8 +1448,6 @@ gog_renderer_finalize (GObject *obj)
 
 	go_line_dash_sequence_free (rend->line_dash);
 	rend->line_dash = NULL;
-	go_line_dash_sequence_free (rend->outline_dash);
-	rend->outline_dash = NULL;
 
 	if (rend->grip_style != NULL) {
 		g_object_unref (rend->grip_style);
